from time import perf_counter

from acb.depends import depends

from ._base import AppBase, AppBaseSettings

main_start = perf_counter()


class AppSettings(AppBaseSettings): ...


class App(AppBase):
    async def post_startup(self) -> None:
        if not self.config.deployed:
            from aioconsole import aprint
            from pyfiglet import Figlet

            fig = Figlet(font="slant", width=90, justify="center")
            await aprint(f"\n\n{fig.renderText(self.config.app.name.upper())}\n")
        if not self.config.debug.production and self.config.deployed:
            self.logger.info("Entering production mode...")

    async def init(self) -> None:
        self.logger.info("Application starting...")

        await self.post_startup()
        main_start_time = perf_counter() - main_start
        self.logger.info(f"App started in {main_start_time} s")

    async def main(self) -> None: ...


depends.set(App)



from logfire import (
    configure,
    instrument_httpx,
    instrument_pydantic,
    instrument_redis,
    instrument_sqlalchemy,
    instrument_system_metrics,
    loguru_handler,
)
from pydantic import SecretStr
from acb.adapters import get_installed_adapters
from acb.depends import depends

from ._base import MonitoringBase, MonitoringBaseSettings


class MonitoringSettings(MonitoringBaseSettings):
    logfire_token: SecretStr = SecretStr("secret")


class Monitoring(MonitoringBase):
    async def init(self) -> None:
        configure(
            token=self.config.monitoring.logfire_token.get_secret_value(),
            service_name=self.config.app.name,
            service_version=self.config.app.version,
        )
        instrument_pydantic(record="all")
        for adapter in [a.name for a in get_installed_adapters()]:
            match adapter:
                case "loguru":
                    self.logger.configure(handlers=[loguru_handler()])
                case "httpx":
                    instrument_httpx()
                case "redis":
                    instrument_redis()
                case "sqlalchemy":
                    sql = depends.get()
                    instrument_sqlalchemy(engine=sql.engine)
                case _:
                    pass
        instrument_system_metrics()


depends.set(Monitoring)



import typing as t

from pydantic import SecretStr, field_validator
from sentry_sdk import init as sentry_init
from sentry_sdk.integrations.asyncio import AsyncioIntegration
from sentry_sdk.integrations.gcp import GcpIntegration
from acb.config import Config
from acb.depends import depends

from ._base import MonitoringBase, MonitoringBaseSettings


class MonitoringSettings(MonitoringBaseSettings):
    sentry_dsn: SecretStr = SecretStr("https://")
    sample_rate: t.Optional[float] = 1.0
    debug: t.Optional[bool] = False
    profiles_sample_rate: t.Optional[float] = 0

    @field_validator("sample_rate", "profiles_sample_rate")
    @classmethod
    def check_sentry_sample_rates(cls, v: float) -> float:  # noqa: F841
        if v > 1 or v < 0:
            raise ValueError("sample rate must be between 0 and 1")
        return v

    @depends.inject
    def __init__(self, config: Config = depends(), **values: t.Any) -> None:
        super().__init__(**values)
        self.sample_rate = self.sample_rate if config.deployed else 1.0


class Monitoring(MonitoringBase):
    async def init(self) -> None:
        sentry_init(
            dsn=self.config.monitoring.sentry_dsn.get_secret_value(),
            server_name=self.config.app.name,
            release=self.config.app.version,
            environment="development" if not self.config.deployed else "production",
            sample_rate=self.config.monitoring.sample_rate,
            debug=self.config.monitoring.debug,
            traces_sample_rate=self.config.monitoring.traces_sample_rate,
            profiles_sample_rate=self.config.monitoring.profiles_sample_rate,
            integrations=[
                GcpIntegration(),
                AsyncioIntegration(),
            ],
        )


depends.set(Monitoring)



import typing as t
from asyncio import sleep
from contextlib import suppress
from warnings import catch_warnings, filterwarnings

from google.api_core.exceptions import BadRequest, Conflict
from google.cloud.dns import Changes
from google.cloud.dns import Client as DnsClient
from google.cloud.dns.resource_record_set import ResourceRecordSet
from google.cloud.dns.zone import ManagedZone
from validators import domain
from validators.utils import ValidationError
from acb.depends import depends

from ._base import DnsBase, DnsBaseSettings, DnsRecord


class DnsSettings(DnsBaseSettings): ...


class Dns(DnsBase):
    current_record_sets: list[ResourceRecordSet] = []
    new_record_sets: list[ResourceRecordSet] = []
    zone: ManagedZone | None = None

    async def init(self) -> None:
        with catch_warnings():
            filterwarnings("ignore", category=Warning)
            self.client = DnsClient(project=self.config.app.project)

    def create_zone(self) -> None:
        self.zone = self.client.zone(self.config.app.name, f"{self.config.app.domain}.")
        if not self.zone.exists():
            self.logger.info(f"Creating cloud_dns zone '{self.config.app.name}...")
            self.zone.create()
            self.logger.info(f"Zone '{self.zone.name}' successfully created")
        else:
            self.logger.info(f"Zone for '{self.zone.name}' exists")

    def list_records(self) -> list[DnsRecord]:
        records = self.zone.list_resource_record_sets()
        records = [
            DnsRecord.model_validate(
                dict(
                    name=record.name,
                    type=record.record_type,
                    ttl=record.ttl,
                    rrdata=record.rrdatas,
                )
            )
            for record in records
        ]
        return records

    async def wait_for_changes(self, changes: Changes) -> None:
        while changes.status != "done":
            await sleep(3)
            changes.reload()

    async def apply_changes(self, changes: Changes):
        try:
            changes.create()
            await self.wait_for_changes(changes)
        except (Conflict, BadRequest) as err:
            change = changes.additions[0]  # type: ignore
            if change.name.split(".")[1] != self.config.app.project:
                raise err
            self.logger.info("Development domain detected - no changes made")

    async def delete_record_sets(self) -> None:
        changes = self.zone.changes()
        for record_set in self.current_record_sets:
            changes.delete_record_set(record_set)
        self.logger.info("Deleting record sets")
        await self.apply_changes(changes)

    async def add_record_sets(self) -> None:
        changes = self.zone.changes()
        for record_set in self.new_record_sets:
            changes.add_record_set(record_set)
        self.logger.info("Creating record sets")
        await self.apply_changes(changes)

    def get_record_set(self, record: DnsRecord) -> ResourceRecordSet:
        return self.zone.resource_record_set(
            name=record.name,
            record_type=record.type,
            ttl=record.ttl,
            rrdatas=record.rrdata,
        )

    def get_current_record(self, record: DnsRecord) -> t.Any:
        current_record = [
            r
            for r in self.list_records()
            if r.name == record.name and r.type == record.type
        ]
        if len(current_record) == 1:
            return current_record[0]

    async def create_records(self, records: list[DnsRecord] | DnsRecord) -> None:
        records = [records] if isinstance(records, DnsRecord) else records
        for record in records:
            if not record.name.endswith("."):
                record.name = f"{record.name}."
            if not isinstance(record.rrdata, list):
                record.rrdata = [record.rrdata]
            for i, r in enumerate(record.rrdata):
                with suppress(ValidationError):
                    if isinstance(r, str) and domain(r) and not r.endswith("."):
                        r = f"{r}."
                        record.rrdata[i] = r
                    if record.type == "TXT":
                        record.rrdata[i] = f'"{r}"'
            current_record = self.get_current_record(record)
            if current_record:
                if current_record.__dict__ == record.__dict__:
                    continue
                self.current_record_sets.append(self.get_record_set(current_record))
                self.logger.info(f"Deleting - {current_record}")
            record_set = self.get_record_set(record)
            self.new_record_sets.append(record_set)
            self.logger.info(f"Creating - {record}")
        if self.current_record_sets:
            await self.delete_record_sets()
        if self.new_record_sets:
            return await self.add_record_sets()
        self.logger.info("No DNS changes detected")


depends.set(Dns)



import typing as t
from functools import cached_property

from pydantic import SecretStr
from acb.depends import depends

from ._base import RequestsBase, RequestsBaseSettings


class RequestsSettings(RequestsBaseSettings):
    base_url: str = ""
    timeout: int = 10
    auth: t.Optional[tuple[str, SecretStr]] = None


class Requests(RequestsBase):
    @cached_property
    def client(self) -> "niquests.AsyncSession":  # type: ignore
        session = niquests.AsyncSession()  # type: ignore
        if self.config.requests.base_url:
            session.base_url = self.config.requests.base_url
        return session

    async def get(
        self,
        url: str,
        timeout: int = 10,
        params: t.Optional[dict[str, t.Any]] = None,
        headers: t.Optional[dict[str, str]] = None,
        cookies: t.Optional[dict[str, str]] = None,
    ) -> t.Any:
        response = await self.client.get(
            url, timeout=timeout, params=params, headers=headers, cookies=cookies
        )
        response.raise_for_status()
        return response

    async def post(
        self,
        url: str,
        data: t.Optional[dict[str, t.Any]] = None,
        timeout: int = 10,
        json: t.Optional[dict[str, t.Any]] = None,
    ) -> t.Any:
        response = await self.client.post(url, data=data, json=json, timeout=timeout)
        response.raise_for_status()
        return response

    async def put(
        self,
        url: str,
        data: t.Optional[dict[str, t.Any]] = None,
        timeout: int = 10,
        json: t.Optional[dict[str, t.Any]] = None,
    ) -> t.Any:
        response = await self.client.put(url, data=data, json=json, timeout=timeout)
        response.raise_for_status()
        return response

    async def delete(self, url: str, timeout: int = 10) -> t.Any:
        response = await self.client.delete(url, timeout=timeout)
        response.raise_for_status()
        return response

    async def patch(
        self,
        url: str,
        timeout: int = 10,
        data: t.Optional[dict[str, t.Any]] = None,
        json: t.Optional[dict[str, t.Any]] = None,
    ) -> t.Any:
        response = await self.client.patch(url, timeout=timeout, data=data, json=json)
        response.raise_for_status()
        return response

    async def head(self, url: str, timeout: int = 10) -> t.Any:
        response = await self.client.head(url, timeout=timeout)
        response.raise_for_status()
        return response

    async def options(self, url: str, timeout: int = 10) -> t.Any:
        response = await self.client.options(url, timeout=timeout)
        response.raise_for_status()
        return response

    async def request(
        self,
        method: str,
        url: str,
        data: t.Optional[dict[str, t.Any]] = None,
        json: t.Optional[dict[str, t.Any]] = None,
        timeout: int = 10,
    ) -> t.Any:
        response = await self.client.request(
            method, url, data=data, json=json, timeout=timeout
        )
        response.raise_for_status()
        return response

    async def close(self) -> None:
        await self.client.close()

    async def init(self) -> None:
        self.logger.debug("Niquests adapter initialized")


depends.set(Requests)



import typing as t
from contextvars import ContextVar
from importlib import import_module, util

from anyio import Path as AsyncPath
from pydantic import BaseModel


class ActionNotFound(Exception): ...


class Action(BaseModel):
    name: str
    pkg: str = "acb"
    module: str = ""
    methods: list[str] = []
    path: AsyncPath = AsyncPath(__file__) / "actions"

    class Config:
        arbitrary_types_allowed = True


action_registry: ContextVar[list[Action]] = ContextVar("action_registry", default=[])


@t.runtime_checkable
class ActionProtocol(t.Protocol):
    def __getattr__(self, item: str) -> Action: ...


class Actions:
    def __getattr__(self, item: str) -> Action:
        if item in self.__dict__:
            return self.__dict__[item]
        raise ActionNotFound(f"Action {item} not found")


actions = Actions()


def create_action(path: AsyncPath) -> Action:
    return Action(
        name=path.stem,
        module=".".join(path.parts[-3:]).removesuffix(".py"),
        pkg=path.parent.parent.parent.stem,
        path=path,
    )


async def register_actions(path: AsyncPath) -> list[Action]:
    actions_path = path / "actions"

    if not await actions_path.exists():
        return []

    found_actions: dict[str, AsyncPath] = {
        a.stem: a
        async for a in actions_path.iterdir()
        if await a.is_dir() and not a.name.startswith("_")
    }

    registry = action_registry.get()
    _actions: list[Action] = []

    for action_name, path in found_actions.items():
        action_index = next(
            (i for i, a in enumerate(registry) if a.name == action_name), None
        )
        if action_index is not None:
            registry.pop(action_index)

        _action = create_action(path=path)
        registry.append(_action)

        try:
            module = import_module(_action.module)
        except ModuleNotFoundError:
            spec = util.spec_from_file_location(_action.path.stem, _action.path)
            if spec and spec.loader:
                module = util.module_from_spec(spec)
                spec.loader.exec_module(module)
            else:
                continue

        if hasattr(module, "__all__"):
            _action.methods = module.__all__
            for attr in [a for a in dir(module) if a in module.__all__]:
                setattr(actions, attr, getattr(module, attr))

        _actions.append(_action)

    return _actions



from pathlib import Path

import typer

cli = typer.Typer()

acb_path = Path(__file__)

app_name = Path.cwd().stem

if Path.cwd() == acb_path:
    raise SystemExit("ACB can not be run in the same directory as ACB itself.")



import asyncio
import os
import signal
import sys
import typing as t
from contextlib import suppress

import pytest

pytest_plugins = ["pytest_asyncio"]


@pytest.fixture(autouse=True)
async def cleanup_async_tasks():
    yield
    tasks = [a for a in asyncio.all_tasks() if a is not asyncio.current_task()]
    if tasks:
        for task in tasks:
            task.cancel()
        await asyncio.gather(*tasks, return_exceptions=True)


def pytest_sessionfinish(session: t.Any, exitstatus: t.Any) -> None:
    sys.stdout.flush()
    sys.stderr.flush()

    def kill_process() -> None:
        os.kill(os.getpid(), signal.SIGTERM)

    asyncio.get_event_loop().call_later(1.0, kill_process)

    with suppress(Exception):
        asyncio.get_event_loop().run_until_complete(asyncio.sleep(0.5))



import typing as t
from unittest.mock import MagicMock, patch

import pytest
from rich.console import Console
from acb.config import Config
from acb.console import console


class TestGetConsole:
    @pytest.fixture
    def mock_config(self) -> t.Generator[MagicMock, None, None]:
        mock_config: MagicMock = MagicMock(spec=Config)
        mock_config.debug = MagicMock()

        with patch(
            "acb.console.depends.inject",
            lambda f: lambda *args, **kwargs: f(*args, config=mock_config, **kwargs),
        ):
            yield mock_config


class TestGetConsoleManager:
    @pytest.fixture
    def mock_config(self) -> t.Generator[MagicMock, None, None]:
        mock_config: MagicMock = MagicMock(spec=Config)
        mock_config.debug = MagicMock()

        with patch(
            "acb.console.depends.inject",
            lambda f: lambda *args, **kwargs: f(*args, config=mock_config, **kwargs),
        ):
            yield mock_config


class TestInitConsole:
    @pytest.fixture
    def mock_config(self) -> t.Generator[MagicMock, None, None]:
        mock_config: MagicMock = MagicMock(spec=Config)
        mock_config.debug = MagicMock()

        with patch(
            "acb.console.depends.inject",
            lambda f: lambda *args, **kwargs: f(*args, config=mock_config, **kwargs),
        ):
            yield mock_config


class TestConsoleGlobal:
    def test_console_global(self) -> None:
        assert isinstance(console, Console)



import typing as t

from hishel import AsyncCacheClient, AsyncRedisStorage, Controller
from hishel._utils import generate_key
from httpcore import Request
from httpx import Response as HttpxResponse
from pydantic import SecretStr
from redis.asyncio import Redis as AsyncRedis
from acb.adapters import import_adapter
from acb.depends import depends

from ._base import RequestsBase, RequestsBaseSettings

Cache = import_adapter()


class RequestsSettings(RequestsBaseSettings):
    base_url: str = ""
    timeout: int = 10
    auth: t.Optional[tuple[str, SecretStr]] = None


class Requests(RequestsBase):
    storage: AsyncRedisStorage
    controller: Controller

    def cache_key(self, request: Request, body: bytes) -> str:
        key = generate_key(request, body)
        return f"{self.config.app.name}:httpx:{key}"

    async def get(
        self,
        url: str,
        timeout: int = 5,
        params: t.Optional[dict[str, t.Any]] = None,
        headers: t.Optional[dict[str, str]] = None,
        cookies: t.Optional[dict[str, str]] = None,
    ) -> HttpxResponse:
        async with AsyncCacheClient(
            storage=self.storage, controller=self.controller
        ) as client:
            return await client.get(
                url, timeout=timeout, params=params, headers=headers, cookies=cookies
            )

    async def post(
        self,
        url: str,
        data: t.Optional[dict[str, t.Any]] = None,
        timeout: int = 5,
        json: t.Optional[dict[str, t.Any]] = None,
    ) -> HttpxResponse:
        async with AsyncCacheClient(
            storage=self.storage, controller=self.controller
        ) as client:
            return await client.post(url, data=data, json=json, timeout=timeout)

    async def put(
        self,
        url: str,
        data: t.Optional[dict[str, t.Any]] = None,
        timeout: int = 5,
        json: t.Optional[dict[str, t.Any]] = None,
    ) -> HttpxResponse:
        async with AsyncCacheClient(
            storage=self.storage, controller=self.controller
        ) as client:
            return await client.put(url, data=data, json=json, timeout=timeout)

    async def delete(self, url: str, timeout: int = 5) -> HttpxResponse:
        async with AsyncCacheClient(
            storage=self.storage, controller=self.controller
        ) as client:
            return await client.delete(url, timeout=timeout)

    async def patch(
        self,
        url: str,
        timeout: int = 5,
        data: t.Optional[dict[str, t.Any]] = None,
        json: t.Optional[dict[str, t.Any]] = None,
    ) -> HttpxResponse:
        async with AsyncCacheClient(
            storage=self.storage, controller=self.controller
        ) as client:
            return await client.patch(url, timeout=timeout, data=data, json=json)

    async def head(self, url: str, timeout: int = 5) -> HttpxResponse:
        async with AsyncCacheClient(
            storage=self.storage, controller=self.controller
        ) as client:
            return await client.head(url, timeout=timeout)

    async def options(self, url: str, timeout: int = 5) -> HttpxResponse:
        async with AsyncCacheClient(
            storage=self.storage, controller=self.controller
        ) as client:
            return await client.options(url, timeout=timeout)

    async def request(
        self,
        method: str,
        url: str,
        timeout: int = 5,
        data: t.Optional[dict[str, t.Any]] = None,
        json: t.Optional[dict[str, t.Any]] = None,
    ) -> HttpxResponse:
        async with AsyncCacheClient(
            storage=self.storage, controller=self.controller
        ) as client:
            return await client.request(
                method, url, timeout=timeout, data=data, json=json
            )

    async def close(self) -> None:
        pass

    async def init(self) -> None:  # type: ignore
        self.logger.debug(self.config.cache)
        self.storage = AsyncRedisStorage(
            client=AsyncRedis(
                host=self.config.cache.host.get_secret_value(),
                port=self.config.cache.port,
            ),
            ttl=self.config.requests.cache_ttl,
        )
        self.controller = Controller(key_generator=self.cache_key)  # type: ignore


depends.set(Requests)



import typing as t

from aiocache import BaseCache
from aiocache.serializers import BaseSerializer
from msgspec import msgpack
from acb.actions.compress import compress, decompress
from acb.config import AdapterBase, Config, Settings
from acb.depends import depends


class CacheBaseSettings(Settings):
    default_ttl: int = 86400
    query_ttl: int = 600
    response_ttl: t.Optional[int] = 3600
    template_ttl: t.Optional[int] = 86400

    @depends.inject
    def __init__(self, config: Config = depends(), **values: t.Any) -> None:
        super().__init__(**values)
        self.response_ttl = self.default_ttl if config.deployed else 1


class MsgPackSerializer(BaseSerializer):
    def __init__(self, *args: t.Any, use_list: bool = True, **kwargs: t.Any) -> None:
        self.use_list = use_list
        super().__init__(*args, **kwargs)

    def dumps(self, value: t.Any) -> bytes:  # type: ignore
        msgpack_data = msgpack.encode(value)
        return compress.brotli(msgpack_data)

    def loads(self, value: bytes | None) -> t.Any:  # type: ignore
        if value is None:
            return None
        msgpack_data = decompress.brotli(value)
        return msgpack.decode(msgpack_data)  # type: ignore


class CacheProtocol(t.Protocol):
    async def set(
        self, key: str, value: bytes, ttl: t.Optional[int] = None
    ) -> None: ...
    async def get(self, key: str) -> t.Optional[bytes]: ...
    async def exists(self, key: str) -> bool: ...
    async def clear(self, namespace: str) -> None: ...
    async def scan(self, pattern: str) -> t.AsyncIterator[str]: ...  # noqa


class CacheBase(BaseCache, AdapterBase): ...



import asyncio
import typing as t
from contextvars import ContextVar
from inspect import currentframe

import nest_asyncio
import rich.repr
from anyio import Path as AsyncPath
from pydantic import BaseModel
from rich import box
from rich.console import RenderableType
from rich.padding import Padding
from rich.table import Table

from .actions import Action, action_registry, register_actions
from .adapters import (
    Adapter,
    _deployed,
    get_adapters,
    register_adapters,
)
from .console import console

nest_asyncio.apply()


@rich.repr.auto
class Pkg(BaseModel, arbitrary_types_allowed=True):
    name: str
    path: AsyncPath
    actions: list[Action] = []
    adapters: list[Adapter] = []


pkg_registry: ContextVar[list[Pkg]] = ContextVar("pkg_registry", default=[])


def register_pkg() -> None:
    path = AsyncPath(currentframe().f_back.f_code.co_filename).parent
    name = path.stem
    registry = pkg_registry.get()
    if name not in [p.name for p in registry]:
        actions = asyncio.run(register_actions(path))
        adapters = asyncio.run(register_adapters(path))
        pkg = Pkg(name=name, path=path, actions=actions, adapters=adapters)
        registry.append(pkg)


register_pkg()

table_args: dict[str, t.Any] = {
    "show_lines": True,
    "box": box.ROUNDED,
    "min_width": 100,
    "border_style": "bold blue",
}


def display_components() -> None:
    if not _deployed:
        pkgs = Table(
            title="[b][u bright_white]A[/u bright_white][bright_green]synchronous "
            "[u bright_white]C[/u bright_white][bright_green]omponent "
            "[u bright_white]B[/u bright_white][bright_green]ase[/b]",
            **table_args,
        )
        for prop in ("Pkg", "Path"):
            pkgs.add_column(prop)
        for i, pkg in enumerate(pkg_registry.get()):
            pkgs.add_row(
                pkg.name,
                str(pkg.path),
                style="bold white" if i % 2 else "bold white on blue",
            )
        pkgs_padded: RenderableType = Padding(pkgs, (2, 4, 0, 4))
        console.print(pkgs_padded)

        actns = Table(
            title="[b][u bright_white]A[/u bright_white][bright_green]ctions",
            **table_args,
        )
        for prop in ("Name", "Pkg", "Methods"):
            actns.add_column(prop)
        for i, action in enumerate(action_registry.get()):
            actns.add_row(
                action.name,
                action.pkg,
                ", ".join(action.methods),
                style="bold white" if i % 2 else "bold white on blue",
            )
        actns_padded: RenderableType = Padding(actns, (0, 4, 1, 4))
        console.print(actns_padded)

        adptrs = Table(
            title="[b][u bright_white]A[/u bright_white][bright_green]datpters",
            **table_args,
        )
        for prop in ("Category", "Name", "Pkg"):
            adptrs.add_column(prop)
        for i, adapter in enumerate(get_adapters()):
            adptrs.add_row(
                adapter.category,
                adapter.name,
                adapter.pkg,
                style="bold white" if i % 2 else "bold white on blue",
            )
        adptrs_padded: RenderableType = Padding(adptrs, (0, 4, 1, 4))
        console.print(adptrs_padded)



import typing as t
from unittest.mock import AsyncMock, MagicMock, patch

import pytest
from pydantic import SecretStr
from acb.config import (
    Config,
    DebugSettings,
    InitSettingsSource,
    PydanticSettingsSource,
    gen_password,
    get_version,
)


@pytest.fixture
def config() -> Config:
    config: Config = Config()
    return config


@pytest.mark.asyncio
async def test_get_version() -> None:
    with patch("acb.config.root_path") as mock_root_path:
        mock_path: MagicMock = MagicMock()
        mock_exists_coro: AsyncMock = AsyncMock()
        mock_exists_coro.return_value = True
        mock_path.exists = mock_exists_coro

        mock_pyproject: MagicMock = MagicMock()
        mock_pyproject.exists = mock_exists_coro
        mock_path.__truediv__ = MagicMock(return_value=mock_pyproject)

        mock_root_path.parent = mock_path

        with patch("acb.config.load") as mock_load:
            mock_toml_coro: AsyncMock = AsyncMock()
            mock_toml_coro.return_value = {"project": {"version": "0.1.0"}}
            mock_load.toml = mock_toml_coro

            version: str = await get_version()
            assert version == "0.1.0"

    with patch("acb.config.root_path") as mock_root_path:
        mock_path: MagicMock = MagicMock()
        mock_exists_coro: AsyncMock = AsyncMock()
        mock_exists_coro.return_value = False
        mock_path.exists = mock_exists_coro

        mock_pyproject: MagicMock = MagicMock()
        mock_pyproject.exists = mock_exists_coro
        mock_path.__truediv__ = MagicMock(return_value=mock_pyproject)

        mock_root_path.parent = mock_path

        version: str = await get_version()
        assert version == "0.1.0"


def test_gen_password() -> None:
    password: str = gen_password()
    assert isinstance(password, str)
    assert password

    password = gen_password(size=20)
    assert isinstance(password, str)
    assert password


@pytest.mark.asyncio
async def test_pydantic_settings_source() -> None:
    settings_cls: MagicMock = MagicMock()
    settings_cls.__name__ = "TestSettings"
    settings_cls.model_fields: dict[str, t.Any] = {}

    source: PydanticSettingsSource = PydanticSettingsSource(settings_cls=settings_cls)  # type: ignore
    assert source.adapter_name == "test"
    assert source.settings_cls == settings_cls

    settings_cls.model_fields = {
        "password": MagicMock(annotation=SecretStr),
        "name": MagicMock(annotation=str),
    }
    secrets: list[str] = source.get_model_secrets()  # type: ignore
    assert "test_password" in secrets
    assert "test_name" not in secrets


@pytest.mark.asyncio
async def test_init_settings_source() -> None:
    settings_cls: MagicMock = MagicMock()
    settings_cls.__name__ = "TestSettings"
    settings_cls.model_fields: dict[str, t.Any] = {}

    init_kwargs: dict[str, t.Any] = {"name": "test", "value": 123}
    source: InitSettingsSource = InitSettingsSource(
        settings_cls=settings_cls,  # type: ignore
        init_kwargs=init_kwargs,
    )

    result: dict[str, t.Any] = await source()
    assert result == init_kwargs


@pytest.mark.asyncio
async def test_debug_settings() -> None:
    debug_settings: DebugSettings = DebugSettings()
    assert not debug_settings.production
    assert not debug_settings.secrets
    assert not debug_settings.logger

    debug_settings = DebugSettings(production=True, secrets=True, logger=True)
    assert debug_settings.production is True
    assert debug_settings.secrets is True
    assert debug_settings.logger is True



import typing as t

from adlfs import AzureBlobFileSystem
from pydantic import SecretStr
from acb.depends import depends

from ._base import StorageBase, StorageBaseSettings


class StorageSettings(StorageBaseSettings):
    connection_string: SecretStr


class Storage(StorageBase):
    file_system: t.Any = AzureBlobFileSystem


depends.set(Storage)



import typing as t
from pathlib import Path
from unittest.mock import AsyncMock, MagicMock, patch

import pytest
from acb.config import Config
from acb.debug import (
    colorized_stderr_print,
    debug,
    get_calling_module,
    init_debug,
    pprint,
    print_debug_info,
)
from acb.logger import Logger


class TestGetCallingModule:
    @pytest.fixture
    def mock_config(self) -> t.Generator[MagicMock, None, None]:
        mock_config: MagicMock = MagicMock(spec=Config)
        mock_config.debug = MagicMock()

        patcher = patch("acb.debug.depends.inject")
        mock_inject = patcher.start()

        def inject_mock_config(func: t.Callable[..., t.Any]) -> t.Callable[..., t.Any]:
            def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                return func(*args, **kwargs, config=mock_config)

            return wrapper

        mock_inject.side_effect = inject_mock_config

        yield mock_config

        patcher.stop()

    def test_get_calling_module_with_debug_disabled(
        self, mock_config: MagicMock
    ) -> None:
        mock_frame: MagicMock = MagicMock()
        mock_frame.f_back.f_back.f_back.f_code.co_filename = "/test/path/test_module.py"

        with patch("acb.debug.logging.currentframe", return_value=mock_frame):
            type(mock_config.debug).test_module = False

            result: t.Any = get_calling_module()

            assert result is None

    def test_get_calling_module_with_exception(self, mock_config: MagicMock) -> None:
        with patch("acb.debug.logging.currentframe", side_effect=AttributeError):
            result: t.Any = get_calling_module()

            assert result is None


class TestPatchRecord:
    @pytest.fixture
    def mock_logger(self) -> t.Generator[MagicMock, None, None]:
        mock_logger: MagicMock = MagicMock(spec=Logger)

        mock_patched_logger: MagicMock = MagicMock()
        mock_logger.patch.return_value = mock_patched_logger

        patcher = patch("acb.debug.depends.inject")
        mock_inject = patcher.start()

        def inject_mock_logger(func: t.Callable[..., t.Any]) -> t.Callable[..., t.Any]:
            def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                return func(*args, **kwargs, logger=mock_logger)

            return wrapper

        mock_inject.side_effect = inject_mock_logger

        yield mock_logger

        patcher.stop()


class TestColorizedStderrPrint:
    def test_colorized_stderr_print_with_asyncio(self) -> None:
        with patch("acb.debug.colorize") as mock_colorize:
            mock_colorize.return_value = "Colored test message"

            with patch("acb.debug.supportTerminalColorsInWindows") as mock_support:
                mock_support.return_value.__enter__ = MagicMock()
                mock_support.return_value.__exit__ = MagicMock()

                mock_coro: AsyncMock = AsyncMock()
                with patch("acb.debug.aprint", return_value=mock_coro) as mock_aprint:

                    def mock_run_impl(coro: t.Any) -> None:
                        return None

                    with patch(
                        "acb.debug.asyncio.run", side_effect=mock_run_impl
                    ) as mock_run:
                        colorized_stderr_print("Test message")

                        mock_colorize.assert_called_once_with("Test message")

                        mock_support.assert_called_once()

                        mock_aprint.assert_called_once_with(
                            "Colored test message", use_stderr=True
                        )

                        mock_run.assert_called_once()

    def test_colorized_stderr_print_with_print(self) -> None:
        with patch("acb.debug.colorize") as mock_colorize:
            mock_colorize.return_value = "Colored test message"

            with patch("acb.debug.supportTerminalColorsInWindows") as mock_support:
                mock_support.return_value.__enter__ = MagicMock()
                mock_support.return_value.__exit__ = MagicMock()

                with patch(
                    "acb.debug.asyncio.run", side_effect=Exception("Test exception")
                ):
                    with patch("sys.stderr") as mock_stderr:
                        with patch("builtins.print") as mock_print:
                            colorized_stderr_print("Test message")

                            mock_print.assert_called_once_with(
                                "Colored test message", file=mock_stderr
                            )


class TestPrintDebugInfo:
    def test_print_debug_info_with_module_not_deployed(self) -> None:
        mock_module: MagicMock = MagicMock(spec=Path)
        with patch("acb.debug.get_calling_module", return_value=mock_module):
            with patch("acb.debug._deployed", False):
                with patch("acb.debug.colorized_stderr_print") as mock_print:
                    result: t.Any = print_debug_info("Test message")

                    mock_print.assert_called_once_with("Test message")

                    assert result is None

    def test_print_debug_info_with_module_deployed(self) -> None:
        mock_module: MagicMock = MagicMock(spec=Path)
        with patch("acb.debug.get_calling_module", return_value=mock_module):
            with patch("acb.debug._deployed", True):
                with patch("acb.debug.patch_record") as mock_patch:
                    result: t.Any = print_debug_info("Test message")

                    mock_patch.assert_called_once_with(mock_module, "Test message")

                    assert result is None

    def test_print_debug_info_without_module(self) -> None:
        with patch("acb.debug.get_calling_module", return_value=None):
            result: t.Any = print_debug_info("Test message")

            assert result is None


class TestPprint:
    @pytest.mark.asyncio
    async def test_pprint(self) -> None:
        with patch("acb.debug.pformat") as mock_pformat:
            mock_pformat.return_value = "Formatted object"

            with patch("acb.debug.aprint") as mock_aprint:
                await pprint({"test": "object"})

                mock_pformat.assert_called_once_with({"test": "object"})

                mock_aprint.assert_called_once_with("Formatted object", use_stderr=True)


class TestInitDebug:
    @pytest.fixture
    def mock_config(self) -> t.Generator[MagicMock, None, None]:
        mock_config: MagicMock = MagicMock(spec=Config)
        mock_config.debug = MagicMock()
        mock_config.debug.production = False
        mock_config.deployed = False

        with patch(
            "acb.debug.depends.inject",
            lambda f: lambda *args, **kwargs: f(*args, config=mock_config, **kwargs),
        ):
            yield mock_config

    def test_init_debug_not_production(self, mock_config: MagicMock) -> None:
        with patch.object(debug, "configureOutput") as mock_configure:
            init_debug()

            mock_configure.assert_called_once()
            assert mock_configure.call_args[1]["prefix"] == "    debug:  "
            assert mock_configure.call_args[1]["includeContext"] is True
            assert "outputFunction" in mock_configure.call_args[1]
            assert "argToStringFunction" in mock_configure.call_args[1]


class TestDebugGlobal:
    def test_debug_global(self) -> None:
        from icecream import ic

        assert debug is ic



import typing as t
from contextlib import suppress
from functools import cached_property

from google.api_core.exceptions import AlreadyExists, PermissionDenied
from google.cloud.secretmanager_v1 import (
    AccessSecretVersionRequest,
    AddSecretVersionRequest,
    CreateSecretRequest,
    DeleteSecretRequest,
    ListSecretsRequest,
    SecretManagerServiceAsyncClient,
)
from acb.config import project
from acb.depends import depends

from ._base import SecretBase, SecretBaseSettings


class SecretSettings(SecretBaseSettings): ...


class Secret(SecretBase):
    project: str = project
    parent: str = f"projects/{project}"

    def extract_secret_name(self, secret_path: str) -> t.Any:
        return secret_path.split("/")[-1].removeprefix(self.prefix)

    async def list(self, adapter: str) -> t.Any:
        request = ListSecretsRequest(
            parent=self.parent, filter=f"{self.prefix}{adapter}_"
        )
        try:
            client_secrets = await self.client.list_secrets(request=request)
        except PermissionDenied:
            raise SystemExit(
                "\n ERROR:  'project' id in 'settings/app.yml' is invalid or not set!\n"
            )
        client_secrets = [
            self.extract_secret_name(secret.name) async for secret in client_secrets
        ]
        return client_secrets

    async def get(self, name: str) -> t.Any:
        path = f"projects/{self.project}/secrets/{name}/versions/latest"
        request = AccessSecretVersionRequest(name=path)
        version = await self.client.access_secret_version(request=request)
        payload = version.payload.data.decode()
        self.logger.info(f"Fetched secret - {name}")
        return payload

    async def create(self, name: str, value: str) -> None:
        with suppress(AlreadyExists):
            request = CreateSecretRequest(
                parent=self.parent,
                secret_id=name,
                secret={"replication": {"automatic": {}}},
            )
            version = await self.client.create_secret(request)
            request = AddSecretVersionRequest(
                parent=version.name,
                payload={"data": value.encode()},
            )
            await self.client.add_secret_version(request)
            self.logger.debug(f"Created secret - {name}")

    async def update(self, name: str, value: str) -> None:
        secret = self.client.secret_path(self.project, name)
        request = AddSecretVersionRequest(
            parent=secret,
            payload={"data": value.encode()},
        )
        await self.client.add_secret_version(request)
        self.logger.debug(f"Updated secret - {name}")

    async def set(self, name: str, value: str) -> None:
        if await self.exists(name):
            await self.update(name, value)
        else:
            await self.create(name, value)

    async def exists(self, name: str) -> bool:
        try:
            await self.get(name)
            return True
        except Exception:
            return False

    async def delete(self, name: str) -> None:
        secret = self.client.secret_path(self.project, name)
        request = DeleteSecretRequest(name=secret)
        await self.client.delete_secret(request=request)
        self.logger.debug(f"Deleted secret - {secret}")

    @cached_property
    def client(self) -> SecretManagerServiceAsyncClient:
        return SecretManagerServiceAsyncClient()

    async def init(self) -> t.NoReturn: ...


depends.set(Secret)



import typing as t
from unittest.mock import MagicMock, patch

import pytest
from acb.config import Config
from acb.depends import depends

T = t.TypeVar("T")


class TestDepends:
    @pytest.fixture
    def mock_config(self) -> t.Generator[MagicMock, None, None]:
        mock_config: MagicMock = MagicMock(spec=Config)
        yield mock_config

    def test_depends_inject_with_no_config(self, mock_config: MagicMock) -> None:
        with patch("acb.depends.inject_dependency") as mock_inject:

            def mock_inject_impl(
                func: t.Callable[..., t.Any],
            ) -> t.Callable[..., t.Any]:
                def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                    return func(*args, **kwargs)

                return wrapper

            mock_inject.side_effect = mock_inject_impl

            @depends.inject
            def my_function(arg1: str, arg2: int) -> tuple[str, int]:
                return arg1, arg2

            result: tuple[str, int] = my_function("test", 123)

            assert result == ("test", 123)

    def test_depends_inject_with_no_args(self, mock_config: MagicMock) -> None:
        with patch("acb.depends.inject_dependency") as mock_inject:

            def mock_inject_impl(
                func: t.Callable[..., t.Any],
            ) -> t.Callable[..., t.Any]:
                def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                    return func(*args, **kwargs)

                return wrapper

            mock_inject.side_effect = mock_inject_impl

            @depends.inject
            def my_function() -> None:
                return None

            result: None = my_function()

            assert result is None

    def test_depends_inject_with_no_args_or_config(
        self, mock_config: MagicMock
    ) -> None:
        with patch("acb.depends.inject_dependency") as mock_inject:

            def mock_inject_impl(
                func: t.Callable[..., t.Any],
            ) -> t.Callable[..., t.Any]:
                def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                    return func(*args, **kwargs)

                return wrapper

            mock_inject.side_effect = mock_inject_impl

            @depends.inject
            def my_function() -> None:
                return None

            result: None = my_function()

            assert result is None

    def test_depends_inject_with_multiple_dependencies_and_no_config(
        self, mock_config: MagicMock
    ) -> None:
        with patch("acb.depends.inject_dependency") as mock_inject:

            def mock_inject_impl(
                func: t.Callable[..., t.Any],
            ) -> t.Callable[..., t.Any]:
                def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                    return func(*args, **kwargs)

                return wrapper

            mock_inject.side_effect = mock_inject_impl

            @depends.inject
            def my_function(arg1: str, arg2: int) -> tuple[str, int]:
                return arg1, arg2

            result: tuple[str, int] = my_function("test", 123)

            assert result == ("test", 123)

    def test_depends_inject_with_multiple_dependencies_and_no_args(
        self, mock_config: MagicMock
    ) -> None:
        with patch("acb.depends.inject_dependency") as mock_inject:

            def mock_inject_impl(
                func: t.Callable[..., t.Any],
            ) -> t.Callable[..., t.Any]:
                def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                    return func(*args, **kwargs)

                return wrapper

            mock_inject.side_effect = mock_inject_impl

            @depends.inject
            def my_function() -> None:
                return None

            result: None = my_function()

            assert result is None

    def test_depends_inject_with_multiple_dependencies_and_no_args_or_config(
        self, mock_config: MagicMock
    ) -> None:
        with patch("acb.depends.inject_dependency") as mock_inject:

            def mock_inject_impl(
                func: t.Callable[..., t.Any],
            ) -> t.Callable[..., t.Any]:
                def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                    return func(*args, **kwargs)

                return wrapper

            mock_inject.side_effect = mock_inject_impl

            @depends.inject
            def my_function() -> None:
                return None

            result: None = my_function()

            assert result is None

    def test_depends_inject_with_multiple_dependencies_and_no_config_or_args(
        self, mock_config: MagicMock
    ) -> None:
        with patch("acb.depends.inject_dependency") as mock_inject:

            def mock_inject_impl(
                func: t.Callable[..., t.Any],
            ) -> t.Callable[..., t.Any]:
                def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                    return func(*args, **kwargs)

                return wrapper

            mock_inject.side_effect = mock_inject_impl

            @depends.inject
            def my_function() -> None:
                return None

            result: None = my_function()

            assert result is None

    def test_depends_inject_with_multiple_dependencies_and_no_config_or_kwargs(
        self, mock_config: MagicMock
    ) -> None:
        with patch("acb.depends.inject_dependency") as mock_inject:

            def mock_inject_impl(
                func: t.Callable[..., t.Any],
            ) -> t.Callable[..., t.Any]:
                def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                    return func(*args, **kwargs)

                return wrapper

            mock_inject.side_effect = mock_inject_impl

            @depends.inject
            def my_function() -> None:
                return None

            result: None = my_function()

            assert result is None

    def test_depends_inject_with_multiple_dependencies_and_no_config_or_args_or_kwargs(
        self, mock_config: MagicMock
    ) -> None:
        with patch("acb.depends.inject_dependency") as mock_inject:

            def mock_inject_impl(
                func: t.Callable[..., t.Any],
            ) -> t.Callable[..., t.Any]:
                def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                    return func(*args, **kwargs)

                return wrapper

            mock_inject.side_effect = mock_inject_impl

            @depends.inject
            def my_function() -> None:
                return None

            result: None = my_function()

            assert result is None

    def test_depends_inject_with_multiple_dependencies_and_no_config_or_args_or_kwargs_or_return(
        self, mock_config: MagicMock
    ) -> None:
        with patch("acb.depends.inject_dependency") as mock_inject:

            def mock_inject_impl(
                func: t.Callable[..., t.Any],
            ) -> t.Callable[..., t.Any]:
                def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                    return func(*args, **kwargs)

                return wrapper

            mock_inject.side_effect = mock_inject_impl

            @depends.inject
            def my_function() -> None:
                return

            result: None = my_function()

            assert result is None

    def test_depends_inject_with_multiple_dependencies_and_no_config_or_args_or_kwargs_or_return_or_type_hints(
        self, mock_config: MagicMock
    ) -> None:
        with patch("acb.depends.inject_dependency") as mock_inject:

            def mock_inject_impl(
                func: t.Callable[..., t.Any],
            ) -> t.Callable[..., t.Any]:
                def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                    return func(*args, **kwargs)

                return wrapper

            mock_inject.side_effect = mock_inject_impl

            @depends.inject
            def my_function() -> None:
                return

            result: None = my_function()

            assert result is None

    def test_depends_inject_with_multiple_dependencies_and_no_config_or_args_or_kwargs_or_return_or_type_hints_or_docstring(
        self, mock_config: MagicMock
    ) -> None:
        with patch("acb.depends.inject_dependency") as mock_inject:

            def mock_inject_impl(
                func: t.Callable[..., t.Any],
            ) -> t.Callable[..., t.Any]:
                def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                    return func(*args, **kwargs)

                return wrapper

            mock_inject.side_effect = mock_inject_impl

            @depends.inject
            def my_function() -> None:
                return

            result: None = my_function()

            assert result is None

    def test_depends_inject_with_multiple_dependencies_and_no_config_or_args_or_kwargs_or_return_or_type_hints_or_docstring_or_body(
        self, mock_config: MagicMock
    ) -> None:
        with patch("acb.depends.inject_dependency") as mock_inject:

            def mock_inject_impl(
                func: t.Callable[..., t.Any],
            ) -> t.Callable[..., t.Any]:
                def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                    return func(*args, **kwargs)

                return wrapper

            mock_inject.side_effect = mock_inject_impl

            @depends.inject
            def my_function() -> None: ...

            result: None = my_function()

            assert result is None

    def test_depends_inject_with_multiple_dependencies_and_no_config_or_args_or_kwargs_or_return_or_type_hints_or_docstring_or_body_or_decorator(
        self, mock_config: MagicMock
    ) -> None:
        with patch("acb.depends.inject_dependency") as mock_inject:

            def mock_inject_impl(
                func: t.Callable[..., t.Any],
            ) -> t.Callable[..., t.Any]:
                def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                    return func(*args, **kwargs)

                return wrapper

            mock_inject.side_effect = mock_inject_impl

            def my_function() -> None: ...

            result: None = depends.inject(my_function)()

            assert result is None

    def test_depends_inject_with_multiple_dependencies_and_no_config_or_args_or_kwargs_or_return_or_type_hints_or_docstring_or_body_or_decorator_or_call(
        self, mock_config: MagicMock
    ) -> None:
        with patch("acb.depends.inject_dependency") as mock_inject:

            def mock_inject_impl(
                func: t.Callable[..., t.Any],
            ) -> t.Callable[..., t.Any]:
                def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
                    return func(*args, **kwargs)

                return wrapper

            mock_inject.side_effect = mock_inject_impl

            def my_function() -> None: ...

            result: t.Callable[..., None] = depends.inject(my_function)

            assert callable(result)



import asyncio
import typing as t
from contextlib import suppress
from inspect import stack

from bevy import dependency, get_repository
from bevy import inject as inject_dependency


@t.runtime_checkable
class DependsProtocol(t.Protocol):
    @staticmethod
    def inject(func: t.Callable[..., t.Any]) -> t.Callable[..., t.Any]: ...

    @staticmethod
    def set(class_: t.Any, instance: t.Any = None) -> t.Any: ...

    @staticmethod
    def get(*args: t.Any) -> t.Any: ...

    def __call__(self, *args: t.Any, **kwargs: t.Any) -> t.Any: ...


class Depends:
    @staticmethod
    def inject(func: t.Callable[..., t.Any]) -> t.Callable[..., t.Any]:
        return inject_dependency(func)

    @staticmethod
    def set(class_: t.Any, instance: t.Any = None) -> t.Any:
        if instance is None:
            return get_repository().set(class_, class_())
        return get_repository().set(class_, instance)

    @staticmethod
    def get(category: t.Any = None) -> t.Any:
        if not category:
            with suppress(AttributeError, IndexError):
                category = [
                    c.strip()
                    for c in (
                        stack()[1][4][0].split("=")[0].strip().lower()  # type: ignore
                    ).split(",")
                ][0].removeprefix("self.")
        class_ = category
        if isinstance(class_, str):
            from .adapters import _import_adapter

            class_ = asyncio.run(_import_adapter(class_))
        return t.cast(class_, get_repository().get(class_))

    def __call__(self, *args: t.Any, **kwargs: t.Any) -> t.Any:
        return dependency()


depends = Depends()



from gzip import GzipFile
from io import BytesIO
from pathlib import Path

import brotli

__all__: list[str] = ["compress", "decompress"]


class Compress:
    @staticmethod
    def gzip(
        content: str | bytes,
        path: str | Path | None = None,
        compresslevel: int = 6,
    ) -> bytes:
        gzip_buffer = BytesIO()
        gz = GzipFile(
            filename=str(path) if path else None,
            mode="wb",
            compresslevel=compresslevel,
            fileobj=gzip_buffer,
        )

        data = content.encode() if isinstance(content, str) else content
        gz.write(data)
        gz.close()
        return gzip_buffer.getvalue()

    @staticmethod
    def brotli(data: bytes | str, level: int = 3) -> bytes:
        if isinstance(data, str):
            data = data.encode()
        return brotli.compress(data, quality=level)


compress = Compress()


class Decompress:
    @staticmethod
    def brotli(data: bytes) -> str:
        return brotli.decompress(data).decode()

    @staticmethod
    def gzip(content: bytes) -> str:
        gzip_buffer = BytesIO(content)
        with GzipFile(fileobj=gzip_buffer, mode="rb") as gz:
            decompressed = gz.read()
        return decompressed.decode()


decompress = Decompress()



import sys
import typing as t
from unittest.mock import AsyncMock, MagicMock, patch

import pytest
from anyio import Path as AsyncPath
from acb.logger import (
    Logger,
    LoggerProtocol,
    LoggerSettings,
)


class Config:
    def __init__(self) -> None:
        self.deployed: bool = False
        self.debug: MagicMock = MagicMock()
        self.debug.production: bool = False
        self.debug.logger: bool = False
        self.root_path: AsyncPath = AsyncPath("/test/path")
        self.logger: t.Any = None


class Settings:
    def __init__(self, **values: t.Any) -> None:
        for key, value in values.items():
            setattr(self, key, value)


mock_config: MagicMock = MagicMock()
mock_config.Config = Config
mock_config.Settings = Settings
mock_depends: MagicMock = MagicMock()

sys.modules["acb.config"] = mock_config
sys.modules["acb.depends"] = mock_depends


class TestLoggerSettings:
    def test_init_default(self) -> None:
        settings: LoggerSettings = LoggerSettings()

        assert not settings.verbose
        assert settings.deployed_level == "WARNING"
        assert settings.log_level == "INFO"
        assert settings.serialize is False
        assert isinstance(settings.format, dict)
        assert isinstance(settings.level_per_module, dict)
        assert isinstance(settings.level_colors, dict)

    def test_init_custom(self) -> None:
        custom_settings: dict[str, t.Any] = {
            "verbose": True,
            "deployed_level": "ERROR",
            "log_level": "DEBUG",
            "serialize": True,
            "format": {
                "time": "[{time}]",
                "level": "{level}",
                "sep": " - ",
                "name": "{name}",
                "line": "({line})",
                "message": "{message}",
            },
            "level_per_module": {"test": "DEBUG"},
            "level_colors": {"debug": "blue"},
        }

        settings: LoggerSettings = LoggerSettings(**custom_settings)

        assert settings.verbose
        assert settings.deployed_level == "ERROR"
        assert settings.log_level == "DEBUG"
        assert settings.serialize
        assert settings.format == custom_settings["format"]
        assert settings.level_per_module == custom_settings["level_per_module"]
        assert settings.level_colors == custom_settings["level_colors"]


class TestLoggerProtocol:
    def test_protocol_compliance(self) -> None:
        logger: Logger = Logger()
        assert isinstance(logger, LoggerProtocol)

        assert hasattr(logger, "debug")
        assert hasattr(logger, "info")
        assert hasattr(logger, "warning")
        assert hasattr(logger, "error")
        assert hasattr(logger, "init")


class TestLogger:
    @pytest.fixture
    def mock_config(self) -> t.Generator[MagicMock, None, None]:
        with patch("acb.logger.depends.get") as mock_get:
            mock_config: MagicMock = MagicMock(spec=Config)
            mock_config.logger = LoggerSettings()
            mock_config.deployed = False
            mock_config.debug = MagicMock()
            mock_config.debug.production = False
            mock_config.debug.logger = False
            mock_config.root_path = "/test/path"

            mock_get.return_value = mock_config

            yield mock_config

    def test_init(self, mock_config: MagicMock) -> None:
        with patch("acb.logger._Logger.__init__") as mock_init:
            Logger()

            mock_init.assert_called_once()
            args: dict[str, t.Any] = mock_init.call_args[1]
            assert "core" in args
            assert args["exception"] is None
            assert args["depth"] == 0
            assert args["record"] is False
            assert args["lazy"] is False
            assert args["colors"] is False
            assert args["raw"] is False
            assert args["capture"] is True
            assert args["patchers"] == []
            assert args["extra"] == {}

    @pytest.mark.asyncio
    async def test_async_sink(self) -> None:
        with patch("acb.logger.aprint") as mock_aprint:
            mock_aprint_coro: AsyncMock = AsyncMock()
            mock_aprint.return_value = mock_aprint_coro

            await Logger.async_sink("Test message")

            mock_aprint.assert_called_once_with("Test message", end="")



import typing as t
from warnings import catch_warnings, filterwarnings

from gcsfs.core import GCSFileSystem
from google.cloud.storage import Client
from acb.config import Config
from acb.depends import depends

from ._base import StorageBase, StorageBaseSettings


class StorageSettings(StorageBaseSettings):
    cors: t.Optional[dict[str, dict[str, list[str] | int]]] = dict(  # noqa
        upload={
            "origin": ["*"],
            "method": ["*"],
            "responseHeader": ["*"],
            "maxAgeSeconds": 600,
        }
    )


class Storage(StorageBase):
    file_system: t.Any = GCSFileSystem

    @staticmethod
    @depends.inject
    def get_client(config: Config = depends()) -> Client:
        with catch_warnings():
            filterwarnings("ignore", category=Warning)
            return Client(project=config.app.project)

    def set_cors(self, bucket_name: str, cors_config: str) -> None:
        bucket = self.get_client().get_bucket(bucket_name)
        bucket.cors = [self.config.storage.cors[cors_config]]
        bucket.patch()
        self.logger.debug(f"CORS policies for {bucket.name!r} bucket set")

    def remove_cors(self, bucket_name: str) -> None:
        bucket = self.get_client().storage_client.get_bucket(bucket_name)
        bucket.cors = []
        bucket.patch()
        self.logger.debug(f"CORS policies for {bucket.name!r} bucket removed")


depends.set(Storage)



import typing as t

from acb.depends import depends

from ._base import SmtpBase, SmtpBaseSettings


class SmtpSettings(SmtpBaseSettings):
    def model_post_init(self, __context: t.Any) -> None:  # noqa: F841
        self.mx_servers = [
            "1 aspmx.l.google.com.",
            "5 alt1.aspmx.l.google.com.",
            "5 alt2.aspmx.l.google.com.",
            "10 alt3.aspmx.l.google.com.",
            "10 alt4.aspmx.l.google.com.",
        ]


class Smtp(SmtpBase): ...


depends.set(Smtp)



import typing as t

from pydantic import SecretStr
from s3fs import S3FileSystem
from acb.config import depends

from ._base import StorageBase, StorageBaseSettings


class StorageSettings(StorageBaseSettings):
    access_key_id: SecretStr
    secret_access_key: SecretStr


class Storage(StorageBase):
    file_system: t.Any = S3FileSystem


depends.set(Storage)



import asyncio
import os
import sys
import typing as t
from contextvars import ContextVar
from importlib import import_module, util
from inspect import stack
from pathlib import Path

import nest_asyncio
import rich.repr
from aioconsole import aprint  # noqa  # type: ignore
from anyio import Path as AsyncPath
from inflection import camelize
from msgspec.yaml import decode as yaml_decode
from pydantic import BaseModel

from ..actions.encode import yaml_encode
from ..depends import depends

nest_asyncio.apply()

root_path: AsyncPath = AsyncPath(Path.cwd())
tmp_path: AsyncPath = root_path / "tmp"
adapters_path: AsyncPath = root_path / "adapters"
actions_path: AsyncPath = root_path / "actions"
settings_path: AsyncPath = root_path / "settings"
app_settings_path: AsyncPath = settings_path / "app.yml"
debug_settings_path: AsyncPath = settings_path / "debug.yml"
adapter_settings_path: AsyncPath = settings_path / "adapters.yml"
secrets_path: AsyncPath = tmp_path / "secrets"
_install_lock: ContextVar[list[str]] = ContextVar("install_lock", default=[])
_deployed: bool = os.getenv("DEPLOYED", "False").lower() == "true"
_testing: bool = os.getenv("TESTING", "False").lower() == "true"
_adapter_import_locks: ContextVar[dict[str, asyncio.Lock]] = ContextVar(
    "_adapter_import_locks", default={}
)
default_adapters = dict(app="default", storage="file", cache="memory")


class AdapterNotFound(Exception): ...


class AdapterNotInstalled(Exception): ...


@t.runtime_checkable
class AdapterProtocol(t.Protocol):
    config: t.Any = None
    logger: t.Any = None

    async def init(self) -> None: ...


@rich.repr.auto
class Adapter(BaseModel, arbitrary_types_allowed=True):
    name: str
    class_name: str
    category: str
    pkg: str = "acb"
    module: str = ""
    enabled: bool = False
    installed: bool = False
    path: AsyncPath = AsyncPath(__file__)

    def __str__(self) -> str:
        return self.__repr__()

    def __hash__(self) -> int:
        return hash((self.name, self.class_name, self.category, self.pkg, self.module))

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, Adapter):
            return False
        return (
            self.name == other.name
            and self.class_name == other.class_name
            and self.category == other.category
            and self.pkg == other.pkg
            and self.module == other.module
        )


adapter_registry: ContextVar[list[Adapter]] = ContextVar("adapter_registry", default=[])

core_adapters = [
    Adapter(
        name="config",
        module="acb.config",
        class_name="Config",
        category="config",
        enabled=True,
        installed=True,
        path=AsyncPath(__file__).parent / "config.py",
    ),
    Adapter(
        name="loguru",
        module="acb.logger",
        class_name="Logger",
        category="logger",
        enabled=True,
        installed=True,
        path=AsyncPath(__file__).parent / "logger.py",
    ),
]
adapter_registry.get().extend([*core_adapters])


def get_adapter(category: str) -> Adapter | None:
    adapter = next(
        (a for a in adapter_registry.get() if a.category == category and a.enabled),
        None,
    )
    return adapter


def get_adapters() -> list[Adapter]:
    return [a for a in adapter_registry.get() if a.enabled]


def get_installed_adapter(category: str) -> Adapter | None:
    adapter = next(
        (a for a in adapter_registry.get() if a.category == category and a.installed),
        None,
    )
    return adapter


def get_installed_adapters() -> list[Adapter]:
    return [a for a in adapter_registry.get() if a.installed]


async def _import_adapter(adapter_category: str) -> t.Any:
    try:
        adapter = get_adapter(adapter_category)
        if adapter is None:
            raise AdapterNotFound(
                f"{adapter_category} adapter not found – check adapters.yml"
            )
    except AttributeError:
        raise AdapterNotFound(
            f"{adapter_category} adapter not found – check adapters.yml"
        )
    try:
        module = import_module(adapter.module)
    except ModuleNotFoundError:
        spec = util.spec_from_file_location(adapter.path.stem, adapter.path)
        module = util.module_from_spec(spec)  # type: ignore
        spec.loader.exec_module(module)
        sys.modules[adapter.name] = module
    adapter_class: t.Any = getattr(module, adapter.class_name)
    if adapter.installed:
        return adapter_class
    async with _adapter_import_locks.get()[adapter_category]:
        if adapter.installed:
            return adapter_class
        try:
            adapter.installed = True
            adapter_settings_class_name = f"{adapter.class_name}Settings"
            adapter_settings_class = getattr(module, adapter_settings_class_name)
            adapter_settings = adapter_settings_class()
            from ..config import Config
            from ..logger import Logger

            config = depends.get(Config)
            setattr(config, adapter_category, adapter_settings)
            instance = depends.get(adapter_class)
            await instance.init()
            logger = depends.get(Logger)
            logger.info(f"Adapter initialized: {adapter.class_name}")
        except Exception as e:
            adapter.installed = False
            raise AdapterNotInstalled(
                f"Failed to install {adapter.class_name} adapter: {e}"
            )
    return adapter_class


async def gather_imports(
    adapter_categories: list[str],
) -> t.Any:
    imports = [_import_adapter(category) for category in adapter_categories]
    _imports = await asyncio.gather(*imports, return_exceptions=True)
    results = []
    for result in _imports:
        if isinstance(result, Exception):
            raise result
        results.append(result)
    return results


def import_adapter(
    adapter_categories: str | list[str] | None = None,
) -> t.Any:
    if isinstance(adapter_categories, str):
        adapter_categories = [adapter_categories]
    if not adapter_categories:
        try:
            adapter_categories = [
                c.strip()
                for c in (
                    stack()[1][4][0].split("=")[0].strip().lower()  # type: ignore
                ).split(",")
            ]
        except (IndexError, AttributeError, TypeError):
            raise ValueError(
                "Could not determine adapter categories from calling context"
            )
    try:
        imported_adapters = asyncio.run(gather_imports(adapter_categories))
    except Exception as e:
        raise AdapterNotInstalled(f"Failed to install adapters: {e}")
    return (
        imported_adapters[0]
        if len(imported_adapters) == 1
        else tuple(imported_adapters)
    )


async def path_adapters(path: AsyncPath) -> dict[str, list[AsyncPath]]:
    return {
        a.stem: [
            m
            async for m in a.iterdir()
            if not m.name.startswith("_") and m.suffix == ".py"
        ]
        async for a in path.iterdir()
        if await a.is_dir() and not a.name.startswith("__")
    }


def create_adapter(
    path: AsyncPath,
    name: str | None = None,
    module: str | None = None,
    class_name: str | None = None,
    category: str | None = None,
    pkg: str | None = None,
    enabled: bool = False,
    installed: bool = False,
) -> Adapter:
    return Adapter(
        path=path,
        name=name or path.stem,
        class_name=class_name or camelize(path.parent.stem),
        category=category or path.parent.stem,
        module=module or ".".join(path.parts[-4:]).removesuffix(".py"),
        pkg=pkg or path.parent.parent.parent.stem,
        enabled=enabled,
        installed=installed,
    )


def extract_adapter_modules(modules: list[AsyncPath], adapter: str) -> list[Adapter]:
    adapter_modules = [create_adapter(p) for p in modules]
    return [a for a in adapter_modules if a.category == adapter]


async def register_adapters(path: AsyncPath) -> list[Adapter]:
    adapters_path = path / "adapters"
    if not await adapters_path.exists():
        return []
    pkg_adapters = await path_adapters(adapters_path)
    adapters: list[Adapter] = []
    for adapter_name, modules in pkg_adapters.items():
        _modules = extract_adapter_modules(modules, adapter_name)
        adapters.extend(_modules)
    if (
        not await adapter_settings_path.exists()
        and not _deployed
        and not _testing
        and root_path.stem != "acb"
    ):
        await settings_path.mkdir(exist_ok=True)
        categories = set()
        categories.update(a.category for a in adapter_registry.get() + adapters)
        await adapter_settings_path.write_bytes(
            yaml_encode(
                {cat: None for cat in categories if cat not in ("logger", "config")}
                | default_adapters,
                sort_keys=True,
            )
        )
    _adapters = (
        {}
        if _testing or root_path.stem == "acb"
        else yaml_decode(await adapter_settings_path.read_text())
    )
    _adapters.update(
        {a.category: None for a in adapters if a.category not in _adapters}
    )
    if not _testing and not root_path.stem == "acb" and not _deployed:
        await adapter_settings_path.write_bytes(yaml_encode(_adapters, sort_keys=True))
    enabled_adapters = {a: m for a, m in _adapters.items() if m}
    for a in [a for a in adapters if enabled_adapters.get(a.category) == a.name]:
        a.enabled = True
    registry = adapter_registry.get()
    for a in adapters:
        module = ".".join(a.module.split(".")[-2:])
        remove = next(
            (_a for _a in registry if ".".join(_a.module.split(".")[-2:]) == module),
            None,
        )
        if remove:
            registry.remove(remove)
        if module in [".".join(_a.module.split(".")[-2:]) for _a in registry]:
            continue
        registry.append(a)
        _adapter_import_locks.get()[a.category] = asyncio.Lock()
    return adapters



import typing as t

from fsspec.implementations.memory import MemoryFileSystem
from acb.depends import depends

from ._base import StorageBase, StorageBaseSettings


class StorageSettings(StorageBaseSettings): ...


class Storage(StorageBase):
    file_system: t.Any = MemoryFileSystem


depends.set(Storage)



from asyncssh import Error, SFTPServer, listen
from pydantic import SecretStr
from acb.depends import depends
from acb.logger import Logger

from ._base import FtpdBase, FtpdBaseSettings


class FtpdSettings(FtpdBaseSettings):
    port: int = 8022
    server_host_keys: SecretStr
    authorized_client_keys: SecretStr


class Ftpd(FtpdBase):
    @depends.inject
    async def init(self, logger: Logger = depends()) -> None:
        try:
            await listen(
                "",
                self,
                server_host_keys=[
                    self.config.ftp_server.server_host_keys.get_secret_value()
                ],
                authorized_client_keys=(
                    self.config.ftp_server.authorized_client_keys.get_secret_value()
                ),
                sftp_factory=SFTPServer,
            )
            logger.info(f"FTP server started on port {self.config.ftpd.port}")
        except (OSError, Error) as exc:
            raise SystemExit(f"\nError starting sftp server: {exc}\n")


depends.set(Ftpd)



import typing as t

from aiocache.backends.memory import SimpleMemoryCache
from aiocache.serializers import PickleSerializer
from acb.depends import depends

from ._base import CacheBase, CacheBaseSettings


class CacheSettings(CacheBaseSettings): ...


class Cache(CacheBase, SimpleMemoryCache):  # type: ignore
    async def init(self, *args: t.Any, **kwargs: t.Any) -> None:
        super().__init__(
            serializer=PickleSerializer(),
            namespace=f"{self.config.app.name}:",
            **kwargs,
        )


depends.set(Cache)



import sys
import typing as t
from pprint import pformat
from re import search

from httpx import Response as HttpxResponse
from acb.actions.encode import load
from acb.adapters import adapter_registry, import_adapter
from acb.adapters.dns._base import DnsRecord
from acb.config import Config
from acb.debug import debug
from acb.depends import depends

from ._base import SmtpBase, SmtpBaseSettings

Dns, Requests = import_adapter()


class SmtpSettings(SmtpBaseSettings):
    @depends.inject
    def __init__(self, config: Config = depends(), **values: t.Any) -> None:
        super().__init__(**values)  # type: ignore
        self.api_url = "https://api.mailgun.net/v3/domains"
        self.mx_servers = ["smtp.mailgun.com"]
        self.api_key = config.smtp.api_key
        self.password = config.smtp.password


class Smtp(SmtpBase):
    requests: Requests = depends()

    async def get_response(
        self,
        req_type: str,
        domain: t.Optional[str] = None,
        data: t.Optional[dict[str, t.Any]] = None,
        params: t.Optional[dict[str, int]] = None,
    ) -> dict[str, t.Any]:
        calling_frame = sys._getframe().f_back.f_code.co_name
        caller = "domain" if search(calling_frame, "domain") else "route"
        url = "/".join(
            [self.config.smtp.api_url, caller, domain or self.config.app.domain or ""]
        )
        data = dict(
            auth=("api", self.config.smtp.api_key.get_secret_value()),
            params=params,
            data=data,
        )
        match req_type:
            case "get":
                resp = await self.requests.get(url)  # type: ignore
            case "put":
                url = "".join([url, "/connection"])
                resp = await self.requests.put(url, data=data)  # type: ignore
            case "post":
                url = "".join([url, "/connection"])
                resp = await self.requests.post(url, data=data)  # type: ignore
            case "delete":
                resp = await self.requests.delete(url)  # type: ignore
            case _:
                raise ValueError
        return await load.json(resp.json())

    async def list_domains(self) -> list[str]:
        resp = await self.get_response("get", params={"skip": 0, "limit": 1000})
        domains = [d.get("name") for d in resp.get("items", {})]
        return domains

    async def get_domain(self, domain: str) -> dict[str, t.Any]:
        return await self.get_response("get", domain=domain)

    async def create_domain(self, domain: str) -> dict[str, t.Any]:
        resp = await self.get_response(
            "post",
            data={
                "name": domain,
                "smtp_password": self.config.smtp.password.get_secret_value(),
                "web_scheme": "https",
            },
        )
        self.logger.debug(resp)
        resp = await self.get_response(
            "put", data={"require_tls": True, "skip_verification": True}
        )
        return resp

    async def delete_domain(self, domain: str) -> dict[str, str]:
        return await self.get_response("delete", domain=domain)

    async def create_domain_credentials(self, domain: str) -> dict[str, str]:
        return await self.get_response(
            "post",
            domain=domain,
            data={
                "login": f"postmaster@{domain}",
                "password": self.config.smtp.password.get_secret_value(),
            },
        )

    async def update_domain_credentials(self, domain: str) -> dict[str, str]:
        return await self.get_response(
            "put",
            domain=domain,
            data={"password": self.config.mail.mailgun.password.get_secret_value()},
        )

    async def get_dns_records(self, domain: str) -> list[DnsRecord]:
        records = await self.get_domain(domain)
        mx_records = records["receiving_dns_records"]
        sending_records = records["sending_dns_records"]
        records = []
        rrdata = []
        for r in mx_records:
            mx_host = f"{r['priority']} {r['value']}."
            rrdata.append(mx_host)
        record = DnsRecord.model_validate(dict(name=domain, type="MX", rrdata=rrdata))
        records.append(record)
        for r in sending_records:
            record = DnsRecord.model_validate(
                dict(name=r["name"], type=r["record_type"], rrdata=r["value"])
            )
            records.append(record)
        debug(records)
        return records

    @depends.inject
    async def create_dns_records(self, dns: Dns = depends()) -> None:
        await self.create_domain(self.config.smtp.domain)
        await self.create_domain_credentials(self.config.smtp.domain)
        records = await self.get_dns_records(self.config.smtp.domain)
        if self.config.mail.mailgun.gmail.enabled:
            await self.delete_domain(self.config.smtp.domain)
            rrdata = self.config.smtp.mx_servers
            record = DnsRecord.model_validate(
                dict(name=self.config.smtp.domain, type="MX", rrdata=rrdata)
            )
            records.append(record)
        else:
            await self.delete_domain(self.config.smtp.domain)
        await dns.create_records(records)

    async def list_routes(self) -> list[t.Any]:
        resp = await self.get_response("get", params={"skip": 0, "limit": 1000})
        domain_routes = [
            r for r in resp["items"] if self.config.smtp.domain in r["expression"]
        ]
        self.logger.debug(pformat(resp["items"]))
        self.logger.debug(len(resp["items"]))
        self.logger.debug(pformat(domain_routes))
        self.logger.debug(len(domain_routes))
        return domain_routes

    async def delete_route(self, route: dict[str, str]) -> HttpxResponse:
        resp = await self.requests.delete("delete", domain={route["id"]})
        self.logger.info(f"Deleted route for {route['description']}")
        return resp

    @staticmethod
    def get_name(address: str) -> str:
        pattern = r"'(.+)@.+"
        name = search(pattern, address)
        return name.group(1) if name else ""

    async def delete_routes(self, delete_all: bool = False) -> None:
        forwards = self.config.smtp.forwards.keys()
        routes = await self.list_routes()
        deletes = []
        deletes.extend(
            [r for r in routes if self.get_name(r["expression"]) not in forwards]
        )
        debug(deletes)
        for f in forwards:
            fs = [r for r in routes if self.get_name(r["expression"]) == f]
            deletes.extend(fs[1:])
        adapter = [
            a for a in adapter_registry.get() if a.category == "email" and a.enabled
        ][0]
        if delete_all or adapter.name == "gmail":
            deletes = [r for r in routes if len(self.get_name(r["expression"]))]
            debug(deletes)
            debug(len(deletes))
        else:
            for d in deletes:
                await self.delete_route(d)

    async def create_route(
        self, domain_address: str, forwarding_addresses: list[str] | str
    ) -> dict[str, str]:
        domain_address = f"{domain_address}@{self.config.smtp.domain}"
        if not isinstance(forwarding_addresses, list):
            forwarding_addresses = [forwarding_addresses]
        actions = ["stop(self)"]
        for addr in forwarding_addresses:
            actions.insert(0, f"forward('{addr}')")
        debug(actions)
        route = {
            "priority": 0,
            "description": domain_address,
            "expression": f"match_recipient('{domain_address}')",
            "action": actions,
        }
        routes = await self.list_routes()
        for r in routes:
            if (r["expression"] == route["expression"]) and (
                r["actions"] == route["action"]
            ):
                self.logger.debug(
                    f"Route for {domain_address}  ==> "
                    f"{', '.join(forwarding_addresses)} exists"
                )
            elif r["expression"] == route["expression"]:
                await self.delete_route(r)
        resp = await self.get_response("post", data=route)
        self.logger.info(
            f"Created route for {domain_address}  ==> "
            f" {', '.join(forwarding_addresses)}"
        )
        return resp

    async def create_routes(self) -> None:
        async for name, forward in self.config.smtp.forwards.items():
            await self.create_route(name, forward)

    async def init(self) -> None:
        await self.create_dns_records()  # type: ignore
        await self.delete_routes()
        await self.create_routes()


depends.set(Smtp)
